def numberToList(number):
    return [int(x) for x in str(number)]

def listToNumber(data, check=False, customErrorMessage = "This data has a string in it"):
    result = 0
    if check:
        for i in data:
            if type(i) == str:
                return customErrorMessage
    for i in data:
        result = result*10 + i
    return result

if __name__ == "__main__":
    print(listToNumber([1, 2, '3', 4], True, ['n','o']))


import math



def makePrimeList(x:int, y:int) -> list:
    sieve = [True] * (y + 1)
    sieve[0] = sieve[1] = False
    for start in range(2, int(math.sqrt(y)) + 1):
        if sieve[start]:
            for multiple in range(start * start, y + 1, start):
                sieve[multiple] = False
        # print(start)
    return [num for num in range(x, y + 1) if sieve[num]]

def makePrimeFile(x:int, y:int, path:str = "/home/marco/1MillionPrimes.txt") -> str:
    sieve = [True] * (y + 1)
    sieve[0] = sieve[1] = False
    for start in range(2, int(math.sqrt(y)) + 1):
        if sieve[start]:
            for multiple in range(start * start, y + 1, start):
                sieve[multiple] = False
    primes = [num for num in range(x, y + 1) if sieve[num]]

    with open(path, "w") as f:
        for i in primes:
            f.write(f"{i}\n")
    
    return path

def isPrime(number):
    if number <= 1:
        return False
    for i in range(2, int((number ** 0.5))):
        if number % i == 0:
            return False
    return True

def findPrimeFactors(n: int, type:str = "set"):
    type = type.lower()
    if type == "set":
        if n == 1:
            return set()
        elif n % 2 == 0:
            return {2} | findPrimeFactors(n//2)
        else:
            d = 3
            limit = int(n**0.5)
            while d <= limit:
                if n % d == 0:
                    return {d} | findPrimeFactors(n//d)
                d += 2
            return {n}
    elif type == "list":
        if n == 1:
            return []
        elif n % 2 == 0:
            return [2] + findPrimeFactors(n//2, "list")
        else:
            d = 3
            limit = int(n**0.5)
            while d <= limit:
                if n % d == 0:
                    return [d] + findPrimeFactors(n//d, "list")
                d += 2
            return [n]
    else:
        return "Invalid Type"
    
    
if __name__ == "__main__":
    print(findPrimeFactors(644))


def generateFigurateNumbers(lower:int, upper:int, type:str) -> list:
    numbers = []
    type = type.lower()
    match type:
        case "triangular":
            for i in range(lower, upper + 1):
                numbers.append(i * (i + 1) // 2)
            return numbers
        case "square":
            for i in range(lower, upper + 1):
                numbers.append(i ** 2)
            return numbers
        case "pentagonal":
            for i in range(lower, upper):
                numbers.append(i * ((3 * i) - 1) // 2)
            return numbers
        case "hexagonal":
            for i in range(lower, upper):
                numbers.append(i * ((2 * i) - 1))
            return numbers
        case "heptagonal":
            for i in range(lower, upper):
                numbers.append(i * ((5 * i) - 3) // 2)
            return numbers
        case "octagonal":
            for i in range(lower, upper):
                numbers.append(i * ((3 * i) - 2))
            return numbers
        case "cubic":
            for i in range(lower, upper):
                numbers.append(i ** 3)
            return numbers
        case "tetrahedal":
            for i in range(lower, upper):
                numbers.append(i * (i + 1) * (i + 2) // 6)
            return numbers
        case _:
            return ["Invalid Type"]

# Tests here
# if __name__ == "__main__":
#     print(generateFigurateNumbers(1, 10, "triangular"))
#     print(generateFigurateNumbers(1, 10, "square"))
#     print(generateFigurateNumbers(1, 10, "pentagonal"))
#     print(generateFigurateNumbers(1, 10, "hexagonal"))
#     print(generateFigurateNumbers(1, 10, "heptagonal"))
#     print(generateFigurateNumbers(1, 10, "octagonal"))
#     print(generateFigurateNumbers(1, 10, "cubic"))
#     print(generateFigurateNumbers(1, 10, "tetrahedal"))




import time

def timer(func): #You can add this as a decorator when defining functions if you want to measure the time it takes to execute.
    def wrapper(*args, **kwargs):
        startTime = time.time()
        func(*args, **kwargs)
        endTime = time.time()
        print(f'Function \"{func.__name__}\" executed in {(endTime - startTime):.4f} seconds')
    return wrapper

class programTimer():
    def __init__(self, verboseOutput = True):
        self.startTime = None
        self.endTime = None
        self.timeElapsed = 0.0
        self.verboseOutput = verboseOutput
        self.finished = False
    
    def start(self):
        self.finished = False
        self.startTime = time.time()
        self.endTime = None
    
    def end(self, handleValue = ""):
        self.finished = True
        self.endTime = time.time()
        self.timeElapsed = self.endTime - self.startTime
        match handleValue:
            case "return":
                return self.timeElapsed
            case "print":
                print(self.timeElapsed)
            case "":
                pass

    def __enter__(self):
        self.finished = False
        self.startTime = time.time()
        return self
    
    def __exit__(self, exc_type, exc_value, traceback):
        self.endTime = time.time()
        self.timeElapsed = self.endTime - self.startTime
        self.finished = True

    def __str__(self): #Also __enter__ and __exit__ methods to be added
        if not self.finished:
            if self.verboseOutput: 
                return(f'Time Elapsed: {(time.time() - self.startTime):.4f} seconds')
            return str(time.time() - self.startTime)
        else:
            if self.verboseOutput:
                return(f'Time Elapsed: {(self.timeElapsed):.4f} seconds')
            return str(self.timeElapsed)

        
    


if __name__ == "__main__":
    import math

    timepassed = programTimer(False)

    # timepass/ed.start()

    def slow_prime_check(n):
        primes = []
        for num in range(2, n):
            is_prime = True
            for i in range(2, int(math.sqrt(num)) + 1):
                # Unnecessary extra checks for no good reason
                if num % i == 0:
                    is_prime = False
                    break
            if is_prime:
                primes.append(num)
    with timepassed as s:
        slow_prime_check(500000)

    print(timepassed)
    # timepassed.end("print")



def pandigital(number):
    number_str = str(number)
    digits = set(number_str)
    target = set(str(i) for i in range(1, len(number_str) + 1))
    return target == digits

if __name__ == "__main__":
    pass